@@ -6,16 +6,19 @@ from .deploy import LocalCluster
 from .diagnostics import progress
 from .client import (Client, Executor, CompatibleExecutor,
                      wait, as_completed, default_client, fire_and_forget,
-                     Future)
+                     Future, futures_of)
+from .lock import Lock
 from .nanny import Nanny
 from .queues import Queue
 from .scheduler import Scheduler
 from .threadpoolexecutor import rejoin
 from .utils import sync
 from .variable import Variable
-from .worker import Worker, get_worker, get_client, secede
+from .worker import Worker, get_worker, get_client, secede, Reschedule
 from .worker_client import local_client, worker_client
 
+from tornado.gen import TimeoutError
+
 from ._version import get_versions
 versions = get_versions()
 __version__ = versions['version']
