@@ -6,6 +6,7 @@ import logging
 from tornado import gen, locks
 from tornado.ioloop import IOLoop
 
+from .config import config
 from .core import CommClosedError
 
 
@@ -50,17 +51,15 @@ class BatchedSend(object):
         self.batch_count = 0
         self.byte_count = 0
         self.next_deadline = None
-        self.recent_message_log = deque(maxlen=100)
+        self.recent_message_log = deque(maxlen=config.get('recent-messages-log-length', 0))
 
     def start(self, comm):
         self.comm = comm
         self.loop.add_callback(self._background_send)
 
-    def __str__(self):
+    def __repr__(self):
         return '<BatchedSend: %d in buffer>' % len(self.buffer)
 
-    __repr__ = __str__
-
     @gen.coroutine
     def _background_send(self):
         while not self.please_stop:
@@ -93,6 +92,8 @@ class BatchedSend(object):
             except Exception:
                 logger.exception("Error in batched write")
                 break
+            finally:
+                payload = None  # lose ref
 
         self.stopped.set()
 
